<?php

namespace Drp\SimpleCms;

use Drp\SimpleCms\Contracts\IsManaged;
use Drp\SimpleCms\Contracts\Previewable;
use Drp\SimpleCms\Contracts\Publishable;
use Drp\SimpleCms\Contracts\ValidatesOnPublish;
use Illuminate\Database\Eloquent\Model;

/**
 * Class Content
 * @package Drp\SimpleCms
 */
abstract class Content extends Model implements Publishable, Previewable, IsManaged, ValidatesOnPublish
{
    use Publisher, Previewer;

    /**
     * Delete all version for the current content.
     *
     * @return integer
     */
    public function destroyAllVersion()
    {
        $versions = $this->where('content_id', $this->content_id)
            ->get()
            ->pluck('id')
            ->toArray();

        return $this->destroy($versions);
    }

    /**
     * Return the latest content for the content type.
     *
     * @return mixed
     */
    public static function latestContent()
    {
        return self::all()
            ->groupBy('content_id')->map(function ($study) {
                return $study->sortBy('status')->first();
            })
            ->flatten();
    }

    /**
     * Returns if the content is visible on the front end of the site.
     *
     * @return boolean
     */
    public function visible()
    {
        return $this->hasPublishedVersion();
    }

    /**
     * Return the content label based on type.
     *
     * @param boolean $plural
     * @param boolean $lowerCase
     * @return string
     * @SuppressWarnings(PHPMD.BooleanArgumentFlag)
     */
    public static function contentLabel($plural = false, $lowerCase = false)
    {
        $label = str_replace('-', ' ', static::LABEL);

        if ($lowerCase) {
            $label = lcfirst($label);
        }

        return $plural ? str_plural($label) : str_singular($label);
    }

    /**
     * Return the label key used to identify the content type.
     *
     * @return string
     */
    public static function contentKey()
    {
        return static::KEY;
    }

    /**
     * Return the name of the contents relationship based on the Key
     *
     * @return string
     */
    public static function relationshipName()
    {
        return lcfirst(studly_case(self::contentKey()));
    }

    /**
     * Return and array of the parameters used to build the admin management links.
     *
     * @return array
     */
    public function getContentRouteParameters()
    {
        return [
            'type' => $this->contentKey(),
            'content_id' => $this->content_id
        ];
    }
    /**
     * Return and array of the parameters used to build the admin management links.
     *
     * @return array
     */
    public function getVersionRouteParameters()
    {
        return [
            'type' => $this->contentKey(),
            'content_id' => $this->id
        ];
    }

    /**
     * Return the url to edit a Study.
     *
     * @return string
     */
    public function adminEditUrl()
    {
        return route($this->adminManageRouteName(), $this->getContentRouteParameters());
    }

    /**
     * Return the manage route name
     *
     * @return string
     */
    public function adminManageRouteName()
    {
        return config('simple_cms.admin_manage_route');
    }
}
